"use strict";
const tslib_1 = require("tslib");
const Asset_1 = require("./Asset");
const HTMLAsset = require("parcel-bundler/lib/assets/HTMLAsset");
const load = require("pug-load");
const lexer = require("pug-lexer");
const parser = require("pug-parser");
const walk = require("pug-walk");
const linker = require("pug-linker");
const generateCode = require("pug-code-gen");
const wrap = require("pug-runtime/wrap");
const filters = require("pug-filters");
// A list of all attributes that should produce a dependency
// Based on https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes
const ATTRS = {
    src: [
        'script',
        'img',
        'audio',
        'video',
        'source',
        'track',
        'iframe',
        'embed'
    ],
    srcset: ['img'],
    href: ['link', 'a'],
    poster: ['video']
};
// A regex to detect if a variable is a 'pure' string (no evaluation needed)
const PURE_STRING_REGEX = /(^"([^"]+)"$)|(^'([^']+)'$)/g;
module.exports = class PugAsset extends Asset_1.Asset {
    constructor(name, pkg, options) {
        super(name, pkg, options);
        this.type = 'html';
    }
    parse(code) {
        let ast = load.string(code, {
            lex: lexer,
            parse: parser,
            filename: this.name
        });
        ast = linker(ast);
        ast = filters.handleFilters(ast);
        return ast;
    }
    collectDependencies() {
        walk(this.ast, node => {
            const recursiveCollect = (cNode) => {
                if (cNode.type === 'Block') {
                    cNode.nodes.forEach((n) => recursiveCollect(n));
                }
                else {
                    if (cNode.filename && cNode.filename !== this.name && !this.dependencies.has(cNode.filename)) {
                        this.addDependency(cNode.filename, {
                            name: cNode.filename,
                            includedInParent: true,
                        });
                    }
                }
            };
            recursiveCollect(node);
            if (node.attrs) {
                for (const attr of node.attrs) {
                    const elements = ATTRS[attr.name];
                    if (node.type === 'Tag' && elements && elements.indexOf(node.name) > -1) {
                        if (PURE_STRING_REGEX.test(attr.val)) {
                            const assetPath = attr.val.substring(1, attr.val.length - 1);
                            this.addURLDependency(assetPath);
                        }
                    }
                }
            }
            return node;
        });
    }
    process() {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield _super("process").call(this);
            const htmlAsset = new HTMLAsset(this.name, this.package, this.options);
            htmlAsset.contents = this.generated.html;
            yield htmlAsset.process();
            Object.assign(this, htmlAsset);
            return this.generated;
        });
    }
    generate() {
        const result = generateCode(this.ast, {
            compileDebug: false,
            pretty: !this.options.minify
        });
        return { html: wrap(result)() };
    }
    shouldInvalidate() {
        return false;
    }
};
//# sourceMappingURL=PugAsset.js.map